from endstone_bstats import SingleLineChart


def test_valid_single_line_chart():
    chart = SingleLineChart("valid_single_line_chart", lambda: 42)
    assert chart.get_chart_data() == {"value": 42}


def test_single_line_chart_data_zero():
    chart = SingleLineChart("single_line_chart_with_none", lambda: 0)
    assert chart.get_chart_data() is None



import logging
import uuid
from typing import Callable

import pytest
import requests_mock

from endstone_bstats import CustomChart, MetricsBase
from endstone_bstats._executor import ScheduledThreadPoolExecutor


class TestMetricsBase(MetricsBase):
    __test__ = False

    @property
    def enabled(self) -> bool:
        return True

    @property
    def service_enabled(self) -> bool:
        return True


class TestCustomChart(CustomChart):
    __test__ = False

    def get_chart_data(self):
        raise Exception("Test exception")


@pytest.fixture
def metrics(mocker):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit")
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit_at_fixed_rate")

    m = TestMetricsBase(
        platform="test_platform",
        server_uuid=uuid.uuid4(),
        service_id=1,
        log_errors=True,
        log_sent_data=True,
        log_response_status_text=True,
    )
    yield m
    m.shutdown()


def test_add_custom_chart(metrics):
    chart = TestCustomChart("custom_chart")
    metrics.add_custom_chart(chart)
    assert chart in metrics._custom_charts


def test_shutdown(mocker, metrics):
    mocker.patch.object(
        ScheduledThreadPoolExecutor, "shutdown", wraps=metrics._executor.shutdown
    )
    metrics.shutdown()
    metrics._executor.shutdown.assert_called_once()


def test_send_data(mocker, metrics):
    mocker.patch("logging.info")
    with requests_mock.Mocker() as rm:
        rm.post("https://bStats.org/api/v2/data/test_platform", status_code=201)
        metrics._submit_data()
        assert rm.last_request.body is not None
        assert logging.info.call_count == 2


def test_submit_data(mocker, metrics):
    mocker.patch.object(MetricsBase, "_send_data")
    mocker.patch.object(
        CustomChart, "_get_request_json_object", side_effect=Exception("Test exception")
    )
    chart = mocker.MagicMock()
    metrics.add_custom_chart(chart)
    metrics._submit_data()
    data = metrics._send_data.call_args.args[0]
    assert data["service"]["id"] == 1


def test_submit_data_with_exception(mocker, metrics):
    mocker.patch.object(
        MetricsBase, "_send_data", side_effect=Exception("Test exception")
    )
    mocker.patch("logging.warning")
    chart = TestCustomChart("custom_chart")
    metrics.add_custom_chart(chart)
    metrics._submit_data()
    assert logging.warning.call_count == 2


def warp_submit(task: Callable, *args, **kwargs):
    task()


def test_start_submitting(mocker, metrics):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit", wraps=warp_submit)
    mocker.patch.object(MetricsBase, "_submit_data")
    metrics._start_submitting()
    metrics._submit_data.assert_called_once()


def test_start_submitting_when_disabled(mocker, metrics):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit", wraps=warp_submit)
    mocker.patch.object(
        TestMetricsBase, "enabled", new_callable=mocker.PropertyMock(return_value=False)
    )
    mocker.patch.object(MetricsBase, "_submit_data")
    mocker.patch.object(MetricsBase, "shutdown", wraps=metrics.shutdown)
    metrics._start_submitting()
    metrics.shutdown.assert_called_once()
    metrics._submit_data.assert_not_called()



import gzip
import json
import logging
import random
import uuid
from abc import ABC, abstractmethod
from io import BytesIO
from typing import Any, Callable, Dict, Set, final

import requests

from endstone_bstats._executor import ScheduledThreadPoolExecutor


class MetricsBase(ABC):
    """
    The MetricsBase class to handle sending metrics to bStats.

    Attributes:
        METRICS_VERSION (str): The version of the Metrics class.
        REPORT_URL (str): The URL to which data is reported.
    """

    METRICS_VERSION = "3.0.3"
    REPORT_URL = "https://bStats.org/api/v2/data/{platform}"

    def __init__(
        self,
        platform: str,
        server_uuid: uuid.UUID,
        service_id: int,
        log_errors: bool,
        log_sent_data: bool,
        log_response_status_text: bool,
    ):
        """
        Initializes the MetricsBase instance.

        Args:
            platform (str): The platform of the service.
            server_uuid (uuid.UUID): The server UUID.
            service_id (int): The service ID.
            log_errors (bool): Whether errors should be logged.
            log_sent_data (bool): Whether the data sent should be logged.
            log_response_status_text (bool): Whether the response status text should be logged.
        """
        self._platform = platform
        self._server_uuid = server_uuid
        self._service_id = service_id
        self._log_errors = log_errors
        self._log_sent_data = log_sent_data
        self._log_response_status_text = log_response_status_text
        self._custom_charts: Set = set()
        self._executor = ScheduledThreadPoolExecutor(max_workers=1)

        if self.enabled:
            self._start_submitting()

    @property
    @abstractmethod
    def enabled(self) -> bool:
        """
        Whether data sending is enabled.
        """

    @property
    @abstractmethod
    def service_enabled(self) -> bool:
        """
        Whether the service is enabled.
        """

    def append_platform_data(self, data: Dict[str, Any]) -> None:
        """
        Append platform-specific data.

        Args:
            data (Dict[str, Any]): The data to append platform-specific values to.
        """
        pass

    def append_service_data(self, data: Dict[str, Any]) -> None:
        """
        Append service-specific data.

        Args:
            data (Dict[str, Any]): The data to append service-specific values to.
        """
        pass

    def submit_task(self, task: Callable[[], None]) -> None:
        """
        Submit the given task

        Args:
            task (Callable[[], None]): The task to be submitted.
        """
        task()

    def log_info(self, message: str) -> None:
        """
        Logs info message.

        Args:
            message (str): The info message.
        """
        logging.info(message)

    def log_error(self, message: str, exception: Exception) -> None:
        """
        Logs error message.

        Args:
            message (str): The error message.
            exception (Exception): The exception that occurred.
        """
        logging.warning(message, exc_info=exception)

    @final
    def add_custom_chart(self, chart: Any):
        """
        Adds a custom chart.

        Args:
            chart: The custom chart to add.
        """
        self._custom_charts.add(chart)

    @final
    def shutdown(self):
        """Shuts down the scheduler."""
        self._executor.shutdown()

    @final
    def _start_submitting(self):
        """
        Starts the submitting process with initial and periodic delays.
        """

        def submit_task():
            if not self.enabled or not self.service_enabled:
                self.shutdown()
                return

            self.submit_task(self._submit_data)

        initial_delay = int((3 + random.random() * 3) * 60)
        second_delay = int((random.random() * 30) * 60)

        self._executor.submit(submit_task, initial_delay)
        self._executor.submit_at_fixed_rate(
            submit_task, initial_delay + second_delay, 60 * 30
        )

    @final
    def _submit_data(self):
        """
        Constructs the JSON data and sends it to bStats.
        """

        platform_data = {}
        self.append_platform_data(platform_data)

        service_data = {}
        self.append_service_data(service_data)

        chart_data = []
        for chart in self._custom_charts:
            try:
                chart_json = chart._get_request_json_object()
                if chart_json is not None:
                    chart_data.append(chart_json)
            except Exception as e:
                if self._log_errors:
                    self.log_error(
                        f"Failed to get data for custom chart with id {chart.chart_id}",
                        e,
                    )

        service_data["id"] = self._service_id
        service_data["customCharts"] = chart_data
        platform_data["service"] = service_data
        platform_data["serverUUID"] = str(self._server_uuid)

        try:
            self._send_data(platform_data)
        except Exception as e:
            if self._log_errors:
                self.log_error("Could not submit bStats metrics data", e)

    @final
    def _send_data(self, data: Dict[str, Any]):
        """
        Sends the JSON data to bStats.

        Args:
            data: The JSON data to send.
        """
        if self._log_sent_data:
            self.log_info(f"Sent bStats metrics data: {data}")

        url = self.REPORT_URL.format(platform=self._platform)
        compressed_data = self._compress(data)

        headers = {
            "Accept": "application/json",
            "Connection": "close",
            "Content-Encoding": "gzip",
            "Content-Length": str(len(compressed_data)),
            "Content-Type": "application/json",
            "User-Agent": "Metrics-Service/1",
        }

        response = requests.post(url, headers=headers, data=compressed_data)
        response.raise_for_status()

        if self._log_response_status_text:
            self.log_info(f"Sent data to bStats and received response: {response.text}")

    @staticmethod
    def _compress(data: Dict[str, Any]) -> bytes:
        """
        Compresses the given data using gzip.

        Args:
            data (dict): The data to be compressed.

        Returns:
            bytes: The compressed data.
        """
        bio = BytesIO()
        with gzip.GzipFile(fileobj=bio, mode="wb") as gzip_file:
            gzip_file.write(json.dumps(data).encode("utf-8"))

        return bio.getvalue()



from __future__ import annotations

from typing import Callable, Dict, List

from endstone_bstats._charts.custom_chart import CustomChart


class AdvancedBarChart(CustomChart):
    def __init__(
        self, chart_id: str, get_values: Callable[[], Dict[str, List[int]] | None]
    ) -> None:
        """
        Class constructor.

        Args:
            chart_id (str): The id of the chart.
            get_values (Callable[[], Dict[str, List[int]] | None]): The callable which is used to request the chart data.
        """
        super().__init__(chart_id)
        self.get_values = get_values

    def get_chart_data(self) -> dict | None:
        """
        Gets the data for the advanced bar chart.

        Returns:
            dict | None: A dictionary with the chart data.
        """
        map_values = self.get_values()
        if not map_values:
            return None

        values = {key: value for key, value in map_values.items() if value}
        if not values:
            return None

        return {"values": values}



from unittest.mock import Mock

from endstone_bstats._charts.multi_line_chart import MultiLineChart


def test_get_chart_data_none():
    """Test that get_chart_data returns None when the callable returns None."""
    callable_mock = Mock(return_value=None)
    multi_line_chart = MultiLineChart("chart_id", callable_mock)

    result = multi_line_chart.get_chart_data()
    assert result is None


def test_get_chart_data_empty():
    """Test that get_chart_data returns None when the callable returns an empty dictionary."""
    callable_mock = Mock(return_value={})
    multi_line_chart = MultiLineChart("chart_id", callable_mock)

    result = multi_line_chart.get_chart_data()
    assert result is None


def test_get_chart_data_valid():
    """Test that get_chart_data processes and returns valid data correctly."""
    data = {"Line A": 10, "Line B": 20, "Line C": 30}
    callable_mock = Mock(return_value=data)
    multi_line_chart = MultiLineChart("chart_id", callable_mock)

    result = multi_line_chart.get_chart_data()
    expected_result = {"values": {"Line A": 10, "Line B": 20, "Line C": 30}}
    assert result == expected_result


def test_get_chart_data_some_zero_values():
    """Test that get_chart_data omits keys with zero values."""
    data = {"Line A": 10, "Line B": 0, "Line C": 30}
    callable_mock = Mock(return_value=data)
    multi_line_chart = MultiLineChart("chart_id", callable_mock)

    result = multi_line_chart.get_chart_data()
    expected_result = {"values": {"Line A": 10, "Line C": 30}}
    assert result == expected_result


def test_get_chart_data_all_zero_values():
    """Test that get_chart_data returns None when all values in the callable result are zero."""
    data = {"Line A": 0, "Line B": 0, "Line C": 0}
    callable_mock = Mock(return_value=data)
    multi_line_chart = MultiLineChart("chart_id", callable_mock)

    result = multi_line_chart.get_chart_data()
    assert result is None



import platform
from pathlib import Path
from typing import Any, Callable, Dict

import psutil
from endstone.plugin import Plugin

from endstone_bstats._base import MetricsBase
from endstone_bstats._charts.drilldown_pie import DrilldownPie
from endstone_bstats._charts.simple_pie import SimplePie
from endstone_bstats._config import MetricsConfig


class Metrics(MetricsBase):
    def __init__(self, plugin: Plugin, service_id: int) -> None:
        """
        Creates a new Metrics instance.

        Args:
            plugin (Plugin): Your plugin instance.
            service_id (int): The id of the service.
                              It can be found at https://bstats.org/what-is-my-plugin-id
        """

        self._plugin = plugin

        # Get the config file
        bstats_folder = Path(plugin.data_folder).parent / "bstats"
        config_file = bstats_folder / "config.toml"
        self._config = MetricsConfig(config_file, True)

        super().__init__(
            platform="bukkit",
            server_uuid=self._config.server_uuid,
            service_id=service_id,
            log_errors=self._config.log_errors_enabled,
            log_sent_data=self._config.log_sent_data_enabled,
            log_response_status_text=self._config.log_response_status_text_enabled,
        )

        self.add_custom_chart(
            SimplePie("endstone_version", lambda: self._plugin.server.version)
        )
        self.add_custom_chart(DrilldownPie("python_version", self._get_python_version))

    @property
    def enabled(self) -> bool:
        return self._config.enabled

    @property
    def service_enabled(self) -> bool:
        return self._plugin.enabled

    def append_platform_data(self, platform_data: Dict[str, Any]) -> None:
        """
        Appends platform-specific data to the provided dict.

        Args:
            platform_data (Dict[str, Any]): The dict to append data to.
        """
        platform_data["playerAmount"] = len(self._plugin.server.online_players)
        # platform_data["onlineMode"] = 1 if Bukkit.get_online_mode() else 0 # TODO: implement this
        platform_data["minecraftVersion"] = self._plugin.server.minecraft_version

        os_name = platform.system()
        if os_name == "Windows":
            platform_data["osName"] = f"Windows {platform.release()}"
            platform_data["osVersion"] = platform.version()
        elif os_name == "Linux":
            platform_data["osName"] = "Linux"
            platform_data["osVersion"] = platform.release()

        platform_data["osArch"] = platform.machine().lower()
        platform_data["coreCount"] = psutil.cpu_count(logical=False)

    def append_service_data(self, service_data: Dict[str, Any]):
        """
        Appends service-specific data to the provided dict.

        Args:
            service_data (Dict[str, Any]): The dict to append data to.
        """
        service_data["pluginVersion"] = self._plugin.description.version

    def submit_task(self, task: Callable[[], None]) -> None:
        self._plugin.server.scheduler.run_task(self._plugin, task)

    def log_info(self, message: str) -> None:
        self._plugin.logger.info(message)

    def log_error(self, message: str, exception: Exception) -> None:
        self._plugin.logger.warning(f"{message}: {exception}")

    def _get_python_version(self) -> dict[str, dict[str, int]]:
        python_impl = platform.python_implementation()
        major, minor, patch = platform.python_version_tuple()
        return {
            f"{python_impl} {major}.{minor}": {
                f"{python_impl} {major}.{minor}.{patch}": 1,
            },
        }



from endstone_bstats import SimplePie


def test_valid_simple_pie():
    chart = SimplePie("valid_simple_pie", lambda: "42")
    assert chart.get_chart_data() == {"value": "42"}


def test_simple_pie_data_empty_string():
    chart = SimplePie("simple_pie_with_empty_string", lambda: "")
    assert chart.get_chart_data() is None


def test_simple_pie_data_none():
    chart = SimplePie("simple_pie_with_none", lambda: None)
    assert chart.get_chart_data() is None



from unittest.mock import Mock

from endstone_bstats._charts.advanced_bar_chart import AdvancedBarChart


def test_get_chart_data_none():
    """Test that get_chart_data returns None when the callable returns None."""
    callable_mock = Mock(return_value=None)
    advanced_bar_chart = AdvancedBarChart("chart_id", callable_mock)

    result = advanced_bar_chart.get_chart_data()
    assert result is None


def test_get_chart_data_empty():
    """Test that get_chart_data returns None when the callable returns an empty dictionary."""
    callable_mock = Mock(return_value={})
    advanced_bar_chart = AdvancedBarChart("chart_id", callable_mock)

    result = advanced_bar_chart.get_chart_data()
    assert result is None


def test_get_chart_data_valid():
    """Test that get_chart_data processes and returns valid data correctly."""
    data = {"Bar A": [10, 20], "Bar B": [15, 25], "Bar C": [30]}
    callable_mock = Mock(return_value=data)
    advanced_bar_chart = AdvancedBarChart("chart_id", callable_mock)

    result = advanced_bar_chart.get_chart_data()
    expected_result = {"values": {"Bar A": [10, 20], "Bar B": [15, 25], "Bar C": [30]}}
    assert result == expected_result


def test_get_chart_data_some_empty_values():
    """Test that get_chart_data omits bars with empty lists."""
    data = {"Bar A": [10, 20], "Bar B": [], "Bar C": [30]}
    callable_mock = Mock(return_value=data)
    advanced_bar_chart = AdvancedBarChart("chart_id", callable_mock)

    result = advanced_bar_chart.get_chart_data()
    expected_result = {"values": {"Bar A": [10, 20], "Bar C": [30]}}
    assert result == expected_result


def test_get_chart_data_all_empty_values():
    """Test that get_chart_data returns None when all bars in the callable result have empty lists."""
    data = {"Bar A": [], "Bar B": [], "Bar C": []}
    callable_mock = Mock(return_value=data)
    advanced_bar_chart = AdvancedBarChart("chart_id", callable_mock)

    result = advanced_bar_chart.get_chart_data()
    assert result is None



from endstone_bstats._base import MetricsBase
from endstone_bstats._charts.advanced_pie import AdvancedPie
from endstone_bstats._charts.custom_chart import CustomChart
from endstone_bstats._charts.drilldown_pie import DrilldownPie
from endstone_bstats._charts.simple_pie import SimplePie
from endstone_bstats._charts.multi_line_chart import MultiLineChart
from endstone_bstats._charts.simple_bar_chart import SimpleBarChart
from endstone_bstats._charts.advanced_bar_chart import AdvancedBarChart
from endstone_bstats._charts.single_line_chart import SingleLineChart
from endstone_bstats._config import MetricsConfig
from endstone_bstats._errors import ChartDataError
from endstone_bstats._metrics import Metrics

__all__ = [
    "Metrics",
    "MetricsBase",
    "MetricsConfig",
    "CustomChart",
    "SimplePie",
    "AdvancedPie",
    "DrilldownPie",
    "SingleLineChart",
    "MultiLineChart",
    "SimpleBarChart",
    "AdvancedBarChart",
    "ChartDataError",
]



from unittest.mock import Mock

from endstone_bstats._charts.drilldown_pie import DrilldownPie


def test_get_chart_data_none():
    """Test that get_chart_data returns None when the callable returns None."""
    callable_mock = Mock(return_value=None)
    drilldown_pie = DrilldownPie("chart_id", callable_mock)

    result = drilldown_pie.get_chart_data()
    assert result is None


def test_get_chart_data_empty():
    """Test that get_chart_data returns None when the callable returns an empty dictionary."""
    callable_mock = Mock(return_value={})
    drilldown_pie = DrilldownPie("chart_id", callable_mock)

    result = drilldown_pie.get_chart_data()
    assert result is None


def test_get_chart_data_valid():
    """Test that get_chart_data processes and returns valid data correctly."""
    data = {
        "Category A": {"Sub A1": 10, "Sub A2": 20},
        "Category B": {"Sub B1": 15, "Sub B2": 25},
        "Category C": {},
    }
    callable_mock = Mock(return_value=data)
    drilldown_pie = DrilldownPie("chart_id", callable_mock)

    result = drilldown_pie.get_chart_data()
    expected_result = {
        "values": {
            "Category A": {"Sub A1": 10, "Sub A2": 20},
            "Category B": {"Sub B1": 15, "Sub B2": 25},
        }
    }
    assert result == expected_result


def test_get_chart_data_some_empty_values():
    """Test that get_chart_data omits empty sub-dictionaries."""
    data = {
        "Category A": {"Sub A1": 10, "Sub A2": 20},
        "Category B": {},
    }
    callable_mock = Mock(return_value=data)
    drilldown_pie = DrilldownPie("chart_id", callable_mock)

    result = drilldown_pie.get_chart_data()
    expected_result = {
        "values": {
            "Category A": {"Sub A1": 10, "Sub A2": 20},
        }
    }
    assert result == expected_result


def test_get_chart_data_all_empty_values():
    """Test that get_chart_data returns None when all sub-dictionaries in the callable result are empty."""
    data = {"Category A": {}, "Category B": {}, "Category C": {}}
    callable_mock = Mock(return_value=data)
    drilldown_pie = DrilldownPie("chart_id", callable_mock)

    result = drilldown_pie.get_chart_data()
    assert result is None



import pytest

from endstone_bstats import Metrics
from endstone_bstats._executor import ScheduledThreadPoolExecutor


@pytest.fixture
def plugin(tmp_path, mocker):
    p = mocker.MagicMock()
    p.data_folder = str(tmp_path)
    p.server.scheduler.run_task = mocker.MagicMock()
    p.enabled = True
    p.logger.warning = mocker.MagicMock()
    p.logger.info = mocker.MagicMock()
    p.server.online_players = ["Player1", "Player2", "Player3"]
    p.description.version = "1.0.0"
    return p


@pytest.fixture
def metrics(mocker, plugin):
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit")
    mocker.patch.object(ScheduledThreadPoolExecutor, "submit_at_fixed_rate")
    m = Metrics(plugin, service_id=1234)
    yield m
    m.shutdown()


def test_enabled(metrics):
    assert metrics.enabled


def test_service_enabled(metrics, plugin):
    assert metrics.service_enabled == plugin.enabled


@pytest.mark.parametrize(
    "os_name, release, version, os_name_expected, os_version_expected",
    [
        (
            "Windows",
            "10",
            "10.0.19041",
            "Windows 10",
            "10.0.19041",
        ),
        (
            "Linux",
            "5.4.0-42-generic",
            "#46-Ubuntu SMP Fri Jul 10 00:24:02 UTC 2020",
            "Linux",
            "5.4.0-42-generic",
        ),
    ],
)
def test_append_platform_data(
    metrics, mocker, os_name, release, version, os_name_expected, os_version_expected
):
    platform_data = {}

    mocker.patch("platform.system", return_value=os_name)
    mocker.patch("platform.release", return_value=release)
    mocker.patch("platform.version", return_value=version)
    mocker.patch("platform.machine", return_value="AMD64")

    metrics.append_platform_data(platform_data)

    assert platform_data["playerAmount"] == 3
    assert platform_data["osName"] == os_name_expected
    assert platform_data["osVersion"] == os_version_expected
    assert platform_data["osArch"] == "amd64"


def test_append_service_data(metrics, plugin):
    service_data = {}
    metrics.append_service_data(service_data)
    assert service_data["pluginVersion"] == plugin.description.version


def test_submit_task(mocker, metrics, plugin):
    task = mocker.MagicMock()
    metrics.submit_task(task)
    plugin.server.scheduler.run_task.assert_called_once_with(plugin, task)


def test_log_info(metrics, plugin):
    message = "Test info message"
    metrics.log_info(message)
    plugin.logger.info.assert_called_once_with(message)


def test_log_error(metrics, plugin):
    message = "Test error message"
    exception = Exception("Test exception")
    metrics.log_error(message, exception)
    plugin.logger.warning.assert_called_once_with(f"{message}: {exception}")


def test_get_python_version(mocker, metrics):
    mocker.patch("platform.python_implementation", return_value="CPython")
    mocker.patch("platform.python_version_tuple", return_value=(3, 12, 4))
    version_data = metrics._get_python_version()
    assert version_data == {"CPython 3.12": {"CPython 3.12.4": 1}}



from unittest.mock import Mock

from endstone_bstats._charts.simple_bar_chart import SimpleBarChart


def test_get_chart_data_none():
    """Test that get_chart_data returns None when the callable returns None."""
    callable_mock = Mock(return_value=None)
    simple_bar_chart = SimpleBarChart("chart_id", callable_mock)

    result = simple_bar_chart.get_chart_data()
    assert result is None


def test_get_chart_data_empty():
    """Test that get_chart_data returns None when the callable returns an empty dictionary."""
    callable_mock = Mock(return_value={})
    simple_bar_chart = SimpleBarChart("chart_id", callable_mock)

    result = simple_bar_chart.get_chart_data()
    assert result is None


def test_get_chart_data_valid():
    """Test that get_chart_data processes and returns valid data correctly."""
    data = {"Bar A": 10, "Bar B": 20, "Bar C": 30}
    callable_mock = Mock(return_value=data)
    simple_bar_chart = SimpleBarChart("chart_id", callable_mock)

    result = simple_bar_chart.get_chart_data()
    expected_result = {"values": {"Bar A": [10], "Bar B": [20], "Bar C": [30]}}
    assert result == expected_result


def test_get_chart_data_some_zero_values():
    """Test that get_chart_data includes zero values wrapped in a list."""
    data = {"Bar A": 10, "Bar B": 0, "Bar C": 30}
    callable_mock = Mock(return_value=data)
    simple_bar_chart = SimpleBarChart("chart_id", callable_mock)

    result = simple_bar_chart.get_chart_data()
    expected_result = {"values": {"Bar A": [10], "Bar B": [0], "Bar C": [30]}}
    assert result == expected_result


def test_get_chart_data_all_zero_values():
    """Test that get_chart_data includes bars with zero values wrapped in a list."""
    data = {"Bar A": 0, "Bar B": 0, "Bar C": 0}
    callable_mock = Mock(return_value=data)
    simple_bar_chart = SimpleBarChart("chart_id", callable_mock)

    result = simple_bar_chart.get_chart_data()
    expected_result = {"values": {"Bar A": [0], "Bar B": [0], "Bar C": [0]}}
    assert result == expected_result
